// pickit.cpp
// ackmed@gotwalls.com
#include <windows.h>
#include "pickit.h"
#include "config.h"
#include "external/LinkedList/LinkedList.h"

// globals
DWORD Debug = DEBUG_OFF;
char DebugBuffer[1024];
char ConfigPath[_MAX_PATH];  // dir where to find pickit.ini
FUNCTIONENTRYPOINTS	*server;
THISGAMESTRUCT *thisgame;
BOOL DumpItems = false;
BOOL inGame = false;		// d2hackit bug/feature work around
DWORD PickupRadiusSquared = 10*10;
BOOL PickupEnabled = true;
LinkedList *Config = new LinkedList;
LinkedList *Items = new LinkedList;

// module info
DWORD	ModuleVersion=MAKELONG(VERSION_MAJOR,VERSION_MINOR);
char	ModuleAuthor[]="ackmed";
char	ModuleWebsite[]="www.gotwalls.com";
char	ModuleDescription[]="pickit for Diablo2 LOD";
char	ModuleEmail[]="ackmed@gotwalls.com";

// module commands
MODULECOMMANDSTRUCT ModuleCommands[]= {

	{	
		"debug",
		OnCommandDebug,
		"debug [devel]c0 toggle debug mode.\n"
		"[devel] added extra debugging"
	},
	{
		"dumpitems",
		OnCommandDumpItems,
		"dumpItemsc0 toggle dumping of item drop packets.\n"
	},
	{
		"radius",
		OnCommandRadius,
		"radius <n>c0 set the pickup radius on items.\n"
		"Default is 10"
	},
	{
		"toggle",
		OnCommandToggle,
		"togglec0 toggles item pickup on/off.\n"
	},
	{
		"reload",
		OnCommandReload,
		"reloadc0 reloads pickit.ini file.\n"
	},
	{NULL}
};

// functions related to the above commands
BOOL PRIVATE OnCommandDebug(char **argv, int argc) {

	// no params passed, just do a simple toggle
	if(argc == 2){
		if(Debug) {
			server->GamePrintString("Debugging disabled");
			Debug = DEBUG_OFF;
		} else {
			server->GamePrintString("Debugging enabled");
			Debug = DEBUG_USER;
		}
		return true;
	} else if(argc == 3) {
		if(strcmpi(argv[2],"devel") == 0) {
			server->GamePrintString("Devel Debugging enabled");
			Debug = DEBUG_DEVEL;
			return true;
		} else if(strcmpi(argv[2],"packet") == 0) {
			server->GamePrintString("Packet Debugging enabled");
			Debug = DEBUG_PACKET;
			return true;
		}
	}
	return false;
}

BOOL PRIVATE OnCommandDumpItems(char** argv, int argc) {

	if(DumpItems) {
		server->GamePrintString("DumpItems Disabled");
	} else {
		server->GamePrintString("DumpItems Enabled");
	}
	DumpItems = 1 - DumpItems;

	return true;
}

BOOL PRIVATE OnCommandRadius(char **argv, int argc) {

	DWORD newRadius;
	char radiusString[64];
	if(argc == 3) {
		newRadius = atoi(argv[2]);
		sprintf(radiusString,"Pickup Radius set to %d",newRadius);
		server->GamePrintString(radiusString);
		PickupRadiusSquared = newRadius * newRadius;
		return true;
	}

	return false;
}

BOOL PRIVATE OnCommandToggle(char **argv, int argc) {

	if(PickupEnabled) {
		server->GamePrintString("Item Pickup disabled.");
	} else {
		server->GamePrintString("Item Pickup enabled.");
	}

	PickupEnabled = 1 - PickupEnabled;
	return true;
}

BOOL PRIVATE OnCommandReload(char** argv, int argc) {

	LinkedItem_t *cfg;

	// delete the old config
	for(cfg = Config->GetFirstItem();cfg != NULL;cfg = Config->RemoveItem(cfg)){
		delete (CONFIGSTRUCT*) cfg->lpData;
	}

	// load new config
	if(!LoadConfig()) {
		server->GamePrintString("Error loading Config");
	} else {
		server->GamePrintString("pickit.ini reloaded");
	}
	return true;
}

// config load function
BOOL LoadConfig(void) {
	char ConfigFile[1024];
	char *Sections;
	char error[1024];
	char *rval;
	int length,counter;
	CONFIGSTRUCT *cfg;

	sprintf(ConfigFile,"%s\\%s",ConfigPath,"pickit.ini");

	if (_access(ConfigFile, 0))
	{ 
		char error[1024];
		sprintf(error, "Unable to open Config: %s", ConfigFile);
		server->GamePrintError(error);
		return false;
	}

	DWORD  alloc=0;
	DWORD  allocStep = 1024;

	while (TRUE) {
		alloc+=allocStep;
		Sections = new char[alloc];
		length = GetPrivateProfileSectionNames(Sections,alloc,ConfigFile);
		
		if(length + 1 != alloc - 1 ) {
			break;
		}
		delete Sections;
	}
	counter = 0;
NEXT:
	while(counter < length) {
		cfg = new CONFIGSTRUCT;


		if(strlen(&Sections[counter]) > 128 || Sections[counter] == '\0') {
			
			if(Debug) {
				sprintf(DebugBuffer,"LoadConfig: Bad Section length (%d)",strlen(&Sections[counter]));
				server->GamePrintInfo(DebugBuffer);
			}
			counter = counter + strlen(&Sections[counter]) + 1;
			delete cfg;
			goto NEXT;
		}

			
		strcpy(cfg->Section,&Sections[counter]);
		counter = counter + strlen(&Sections[counter]) + 1;
		// now query the section and fill in data
		
		// ItemCode should be 3 or 4 chars long
		rval = server->GetHackProfileString("pickit", cfg->Section, "Code");
		if(strlen(rval)) {
			if(strlen(rval) != 4 && strlen(rval) != 3) {
				sprintf(error,"Bad Code value '%s' in section [%s], Skipping",rval,cfg->Section);
				server->GamePrintString(error);
				delete cfg;
				goto NEXT;
			}
			
			// if 4 char is a space, chop it off
			if(rval[3] == ' ') 
				rval[3] = '\0';

			strcpy(cfg->item.ItemCode,rval);
			// delete rval;
		} else {
			cfg->item.ItemCode[0] = '\0';
		}

		// Item Description
		rval = server->GetHackProfileString("pickit", cfg->Section, "Description");
		if(strlen(rval)) {
			if(strlen(rval) > 127) {
				sprintf(error,"Description too long in section [%s], Skipping",cfg->Section);
				server->GamePrintString(error);
				delete cfg;
				goto NEXT;
			}
			strcpy(cfg->Description,rval);
			// delete rval;
		} else {
			cfg->Description[0] = '\0';
		}

		// identified
		rval = server->GetHackProfileString("pickit", cfg->Section, "Identified");
		if(strlen(rval)) {
			cfg->item.isIdentified = atoi(rval);
			// delete rval;
		} else {
			cfg->item.isIdentified = ITEM_UNSET;
		}

		// ethereal
		rval = server->GetHackProfileString("pickit", cfg->Section, "isEthereal");
		if(strlen(rval)) {
			cfg->item.isEthereal = atoi(rval);
			// delete rval;
		} else {
			cfg->item.isEthereal = ITEM_UNSET;
		}

		// item should have sockets, doesnt matter how many unlesss NumSockets is used
		rval = server->GetHackProfileString("pickit", cfg->Section, "HasSockets");
		if(strlen(rval)) {
			cfg->item.hasSockets = atoi(rval);
			// delete rval;
		} else {
			cfg->item.hasSockets = ITEM_UNSET;
		}

		// dont pickup item.  useful if you only want to be told of the item drop
		rval = server->GetHackProfileString("pickit", cfg->Section, "Pickup");
		if(strlen(rval)) {
			cfg->Pickup = atoi(rval);
		//	delete rval;
		} else {
			cfg->Pickup = CONFIG_UNSET;
		}
		
		// item level
		rval = server->GetHackProfileString("pickit", cfg->Section, "Level");
		if(strlen(rval)) {
			if(strcmpi(rval,"rare") == 0) {
				cfg->item.ItemLevel = ITEM_LEVEL_RARE;
			} else if(strcmpi(rval,"unique") == 0) {
				cfg->item.ItemLevel = ITEM_LEVEL_UNIQUE;
			} else if(strcmpi(rval,"set") == 0) {
				cfg->item.ItemLevel = ITEM_LEVEL_SET;
			} else if(strcmpi(rval,"magic") == 0) {
				cfg->item.ItemLevel = ITEM_LEVEL_MAGIC;
			} else if(strcmpi(rval,"craft") == 0) {
				cfg->item.ItemLevel = ITEM_LEVEL_CRAFT;
			} else if(strcmpi(rval,"normal") == 0) {
				cfg->item.ItemLevel = ITEM_LEVEL_NORMAL;
			} else if(strcmpi(rval,"superior") == 0) {
				cfg->item.ItemLevel = ITEM_LEVEL_SUPERIOR;
			} else if(strcmpi(rval,"lowquality") == 0) {
				cfg->item.ItemLevel = ITEM_LEVEL_INFERIOR;
			} else {
				sprintf(error,"Bad Level '%s' in section [%s], skipping",rval,cfg->Section);
				server->GamePrintString(error);
				// delete rval;
				delete cfg;
				goto NEXT;
			}
			// delete rval;
		} else {
			cfg->item.ItemLevel = ITEM_UNSET;
		}
	
		// item class
		rval = server->GetHackProfileString("pickit", cfg->Section, "Type");
		if(strlen(rval)) {
			if(strcmpi(rval,"Helm") == 0) {
				cfg->item.ItemType = ITEM_ITEMTYPE_HELM;
			} else if(strcmpi(rval,"Armor") == 0) {
				cfg->item.ItemType = ITEM_ITEMTYPE_ARMOR;
			} else if(strcmpi(rval,"Weapon") == 0) {
				cfg->item.ItemType = ITEM_ITEMTYPE_WEAPON;
			} else if(strcmpi(rval,"Bow") == 0) {
				cfg->item.ItemType = ITEM_ITEMTYPE_BOW;
			} else if(strcmpi(rval,"Shield") == 0) {
				cfg->item.ItemType = ITEM_ITEMTYPE_SHIELD;
			} else if(strcmpi(rval,"Other") == 0) {
				cfg->item.ItemType = ITEM_ITEMTYPE_OTHER;
			} else {
				sprintf(error,"Bad class '%s' in section [%s], skipping",rval,cfg->Section);
				server->GamePrintString(error);
				// delete rval;
				delete cfg;
				goto NEXT;
			}
			// delete rval;
		} else {
			cfg->item.ItemType = ITEM_UNSET;
		}
		Config->AddItem(cfg);

		if(Debug == DEBUG_DEVEL) {
			sprintf(DebugBuffer,"[%s] Des:'%s' Code:'%s' %d %d %d %d",cfg->Section,cfg->Description,cfg->item.ItemCode,cfg->Pickup,cfg->item.hasSockets,cfg->item.isIdentified,cfg->item.ItemLevel,cfg->item.ItemType);
			server->GamePrintInfo(DebugBuffer);
		}

	}
	if(Debug == DEBUG_DEVEL) {
		sprintf(DebugBuffer,"LoadConfig: %d sections loaded.",Config->GetItemCount());
		server->GamePrintInfo(DebugBuffer);
	}
	delete Sections;
	return true;
}

DWORD GetDistanceSquared(DWORD x1, DWORD y1, DWORD x2, DWORD y2) {
	return ( ((x1 - x2) * (x1 - x2)) + ((y1 - y2) * (y1 - y2)));
}

// functions called by d2hackit
BOOL EXPORT OnClientStart() {
	return LoadConfig();
}

BOOL EXPORT OnClientStop() {
	return true;
}

VOID EXPORT OnGameJoin(THISGAMESTRUCT* aThisgame) {
	thisgame = aThisgame;
	LinkedItem_t *linkeditem;
	
	// make sure there are now stale items in the llist
	for(linkeditem = Items->GetFirstItem();linkeditem != NULL;linkeditem = Items->RemoveItem(linkeditem)){
		delete (ITEMSTRUCT*) linkeditem->lpData;
	}
/*	
	FILE *stream; 
	char t[1024]; 
	sprintf(t, "OnGameLeave Called"); 
	stream = fopen("c:\\log.d2", "a"); 
	fprintf(stream, "%s\n", t); 
	fclose(stream); 
*/	
	inGame = true; // d2hackit bug/feature work around
}

VOID EXPORT OnGameLeave(THISGAMESTRUCT* aThisgame) {
/*	
	FILE *stream; 
	char t[1024]; 
	sprintf(t, "OnGameLeave Called"); 
	stream = fopen("c:\\log.d2", "a"); 
	fprintf(stream, "%s\n", t); 
	fclose(stream); 
*/	
	inGame = false;  // d2hackit bug/feature work around
}

// is run ~every 1/10th a second
DWORD EXPORT OnGameTimerTick(void)
{
	DWORD HalfPickupRadiusSquared;
	DWORD DistanceSquared;
	LinkedItem_t *linkeditem;
	ITEMSTRUCT *item;
	BYTE PickupPacket[13];
	static DWORD counter;

/*	
	FILE *stream; 
	char t[1024]; 
	sprintf(t, "OnGameTimerTick Called"); 
	stream = fopen("c:\\log.d2", "a"); 
	fprintf(stream, "%s\n", t); 
	fclose(stream); 
*/	
	// d2hackit bug/feature work around
	// OnGameTimerTick() gets called even when not in game.
	if(!inGame) {
		return true;
	}

	// item pickup is off
	if(!PickupEnabled) {
		return true;
	}

	HalfPickupRadiusSquared = PickupRadiusSquared / 4;

	// if item is less then 1/2 pickupradius away send pickup request everytime
	// if item is greater then 1/2 pickupradius away send pickup reqest every 4 function calls
	// ignore items outside pickup radius
	
	for(linkeditem = Items->GetFirstItem();linkeditem != NULL;linkeditem = Items->GetNextItem(linkeditem)) {
		item = (ITEMSTRUCT*)linkeditem->lpData;
		DistanceSquared = GetDistanceSquared(thisgame->player->PlayerPositionX,thisgame->player->PlayerPositionY,item->PositionX,item->PositionY);

		if(DistanceSquared <= HalfPickupRadiusSquared || (DistanceSquared <= PickupRadiusSquared && counter == 3)) {
			
			PickupPacket[0] = 0x16;
			PickupPacket[1] = 0x04;
			PickupPacket[2] = PickupPacket[3] = PickupPacket[4] = 0;
			int ItemID = item->ItemID;
			memcpy(&PickupPacket[5],&ItemID,4);
			PickupPacket[9] = PickupPacket[10] = PickupPacket[11] = PickupPacket[12] = 0;
			server->GameSendPacketToServer(PickupPacket, 13);
			if(Debug == DEBUG_DEVEL) {
				sprintf(DebugBuffer,"Sending Pickup Request (%d)",ItemID);
				server->GamePrintInfo(DebugBuffer);
			}
		}
	}

	counter++;
	if(counter > 3) {
		counter = 0;
	}

	return true;
}

DWORD EXPORT OnGamePacketBeforeReceived(BYTE* aPacket, DWORD aLen) {

	ITEMSTRUCT *item;  
	CONFIGSTRUCT *cfg;
	LinkedItem_t *linkedcfg;
	LinkedItem_t *linkeditem;
	BOOL DoPickup;
	

	// see if the packet is has the right messageID
	if(aPacket[0] == ITEM_MESSAGEID_DROP) {
		item = ParseItemDrop(aPacket,aLen);
		if(item == NULL) {
			// adds alot of un-needed spam
		/*	if(Debug == DEBUG_DEVEL) {
				server->GamePrintInfo("OnGamePacketBeforeReceived:  ParseItemDrop returnned NULL");
			}
			*/
			return aLen;
		}

		// need to make sure we delete item if not used

		if(DumpItems) {
			DumpItemStruct(item);
		}

		// see if we should pickup/annouce this item
		for(linkedcfg = Config->GetFirstItem();linkedcfg != NULL;linkedcfg = Config->GetNextItem(linkedcfg)){
			DoPickup = true;
			cfg = (CONFIGSTRUCT*)linkedcfg->lpData;
			
			// hasSockets
			if(cfg->item.hasSockets != ITEM_UNSET && item->hasSockets != cfg->item.hasSockets)
				DoPickup = false;

			// isIdentified
			if(cfg->item.isIdentified != ITEM_UNSET && item->isIdentified != cfg->item.isIdentified)
				DoPickup = false;

			if(cfg->item.isEthereal != ITEM_UNSET && item->isEthereal != cfg->item.isEthereal)
				DoPickup = false;

			// ItemCode 
			if(cfg->item.ItemCode[0] != '\0' && strcmp(cfg->item.ItemCode,item->ItemCode) != 0) 
				DoPickup = false;
			
			// ItemType
			if(cfg->item.ItemType != ITEM_UNSET && item->ItemType != cfg->item.ItemType) 
				DoPickup = false;

			// ItemLevel
			if(cfg->item.ItemLevel != ITEM_UNSET && item->ItemLevel != cfg->item.ItemLevel)
				DoPickup = false;

			// want to pick or be notified of this item
			if(DoPickup) {
				if(Debug) {
					sprintf(DebugBuffer,"Pickup/Noticed cause by [%s]",cfg->Section);
					server->GamePrintInfo(DebugBuffer);
				}
				

				// do notification if description exists;
				if(strlen(cfg->Description)) {
					// if outside our pickup radius add directional information
					if(GetDistanceSquared(thisgame->player->PlayerPositionX,thisgame->player->PlayerPositionY,item->PositionX,item->PositionY) > PickupRadiusSquared) {
						
						int x = thisgame->player->PlayerPositionX - item->PositionX;
						int y = thisgame->player->PlayerPositionY - item->PositionY;
						char DesBuffer[1024];

						double dir;
						double dirSetting = 0.414;  // tan of 22.5 degrees
						if( abs(x) > abs(y)) {
							dir = (float)y / x;
						} else {
							dir = (float)x / y;
						}
						
						if(x > 0 && y > 0) {
							if(dir < dirSetting) {
								if(x > y) {
									sprintf(DesBuffer,"%s (NorthWest)",cfg->Description);
								} else {
									sprintf(DesBuffer,"%s (NorthEast)",cfg->Description);
								}
							} else {
								sprintf(DesBuffer,"%s (North)",cfg->Description);
							}
						} else if(x > 0 && y < 0) {
							if(dir > -dirSetting) {
								if(x > abs(y)) {
									sprintf(DesBuffer,"%s (NorthWest)",cfg->Description);
								} else {
									sprintf(DesBuffer,"%s (SouthWest)",cfg->Description);
								}
							} else {
								sprintf(DesBuffer,"%s (West)",cfg->Description);
							}
						} if(x < 0 && y > 0) {
							if(dir > -dirSetting) {
								if(abs(x) > y) {
									sprintf(DesBuffer,"%s (SouthEast)",cfg->Description);
								} else {
									sprintf(DesBuffer,"%s (NorthEast)",cfg->Description);
								}
							} else {
								sprintf(DesBuffer,"%s (East)",cfg->Description);
							}
						} else if(x < 0 && y < 0) {
							if( dir < dirSetting) {
								if(abs(x) > abs(y)) {
									sprintf(DesBuffer,"%s (SouthEast)",cfg->Description);
								} else {
									sprintf(DesBuffer,"%s (SouthWest)",cfg->Description);
								}
							} else {
								sprintf(DesBuffer,"%s (South)",cfg->Description);
							}
						}
						server->GamePrintString(DesBuffer);
					} else {
						server->GamePrintString(cfg->Description);
					}
				}
				if(cfg->Pickup != false) {
					Items->AddItem(item);
				} else {
					delete item;
				}
				return aLen;
			} 
		}
		return aLen;
	} else if(aPacket[0] == 0x0a && aPacket[1] == 0x04) {  // item was picked up or is gone
		BitFields Pickedup(aPacket,aLen);
		if(Debug == DEBUG_PACKET) {
			sprintf(DebugBuffer,"%s","RECV: ");
			for(int i = 0;i < aLen;i++) {
				sprintf(DebugBuffer,"%s %.2x",DebugBuffer,aPacket[i]);
			}
			server->GamePrintInfo(DebugBuffer);
		}
		Pickedup.GetField(16);
		DWORD ItemID = Pickedup.GetField(32);

		for(linkeditem = Items->GetFirstItem();linkeditem != NULL;linkeditem = Items->GetNextItem(linkeditem)) {
			item = (ITEMSTRUCT*)linkeditem->lpData;
			if(item->ItemID == ItemID) {
				if(Debug == DEBUG_DEVEL) {
					sprintf(DebugBuffer,"Item Pickup/Disappear (%d)\n",item->ItemID);
					server->GamePrintInfo(DebugBuffer);
				}
				// remove item and its place in the llist
				delete item;
				Items->RemoveItem(linkeditem);
				return aLen;
			}
		}

	} else if(aPacket[0] == 0x2c && aPacket[1] == 0x00 && PickupEnabled) {
		if(aLen == 8 && aPacket[6] == 0x16) {
			server->GamePrintString("Inventory Full! Auto-pickup disabled");
			PickupEnabled = false;
		}
	}

	return aLen;
}

// 
// stuff below here is from ClientCore.cpp
//////////////////////////////////////////////////////////////////////
// Dll entry/exit
//////////////////////////////////////////////////////////////////////
BOOL APIENTRY DllMain(HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
	BOOL hResult = TRUE;
	char *t;
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			
			// Create server struct			
			server = new FUNCTIONENTRYPOINTS; 

			// Bind exported functions from server
			HMODULE hModule;
			hModule = (HMODULE)GetModuleHandle("D2HackIt");
			
			// Macros make this look a lot nicer :)
			BIND_TO_SERVER(GamePrintInfo);
			BIND_TO_SERVER(GamePrintError);
			BIND_TO_SERVER(GamePrintVerbose);
			BIND_TO_SERVER(GamePrintString);
			BIND_TO_SERVER(GameCommandLine);
			BIND_TO_SERVER(GameSendPacketToServer);
			BIND_TO_SERVER(GameSendPacketToGame);
			BIND_TO_SERVER(GetFingerprint);
			BIND_TO_SERVER(Intercept);
			BIND_TO_SERVER(GetHackProfileString);
			BIND_TO_SERVER(SetHackProfileString);
			BIND_TO_SERVER(GetThisgameStruct);

			// Get plugin path
			t = ConfigPath;
			if (GetModuleFileName((HINSTANCE)hModule, t, _MAX_PATH)) {
				int p=strlen(ConfigPath);
				while (p) {
						if (ConfigPath[p] == '\\')
							{ ConfigPath[p] = 0; p=0;}
						else
					p--;
				}
			}	 
			// initiate client
			// hResult = OnClientStart();
			break;

		case DLL_PROCESS_DETACH:
			// kill client

			// hResult = OnClientStop();

			delete server;
			break;
    } 
    return hResult;
}

//////////////////////////////////////////////////////////////////////
// Stubfunctions for 'property get' functions.
//////////////////////////////////////////////////////////////////////
LPCSTR	EXPORT GetModuleAuthor()		{return ModuleAuthor;}
LPCSTR	EXPORT GetModuleWebsite()		{return ModuleWebsite;}
DWORD	EXPORT GetModuleVersion()		{return ModuleVersion;}
LPCSTR	EXPORT GetModuleEmail()			{return ModuleEmail;}
LPCSTR	EXPORT GetModuleDescription()	{return ModuleDescription;}

//////////////////////////////////////////////////////////////////////
// OnClientCommandLine
// -------------------------------------------------------------------
// The modules own extension of the command line interface. Any custom
// commands you add are parsed here.
//
// Return value should be TRUE, but it is not used at this 
// time.
//
// Arguments when we get here:
// argv[0]			Name of module
// argv[1]			Name of command (If supplied)
// argv[2 ... n]	The rest
//
// Syntax in the game: .<module> <arguments>
//////////////////////////////////////////////////////////////////////
BOOL EXPORT OnGameCommandLine(char* argv[], int argc)
{
	// Check if user supplied anything at all, if not assume help...
	if (argc==1)
		argv[argc++]="help";


	MODULECOMMANDSTRUCT* mcs=ModuleCommands;

	while (mcs->szName) {
		if (!stricmp(mcs->szName, argv[1]))
			break;
		mcs++;
	}

	char *p,*t,*msg,*fMsg;
	fMsg=new char[256];
	//
	// Is this a built-in function ?
	if (mcs->szName) {
		//
		// If functions returns false, show usage help
		if (!mcs->pFunc(argv, argc)) {
			t=new char[strlen(mcs->szUsage)+1];
			server->GamePrintInfo("Usage:");
			sprintf((char*)t, "%s", mcs->szUsage);
			if (strlen((char*)t))
			{
				msg=p=t;
				while (*p != 0) {
					if (*p == '\n') 
					{
						*(p++) = 0;
						sprintf(fMsg, "c4.%s %s", argv[0], msg);
						server->GamePrintInfo((char*)fMsg);
					if (*p != 0)
						msg = p;
					} else
						p++;
					}
				sprintf(fMsg, "c4.%s %s", argv[0], msg);
				server->GamePrintInfo((char*)fMsg);
			}
			delete t;
		}
	} else {
	// Unknown command, show catch-all help phraze.
	t=new char[128];
	sprintf(t, "Unknown command c4'%s %s'c0 - try c4'.%s help'c0 to get help.",
		argv[0], argv[1], argv[0]);
	server->GamePrintError(t);
	delete t;
	}
	delete fMsg;
	return TRUE;
}


//////////////////////////////////////////////////////////////////////
// OnClientCommandHelp
// -------------------------------------------------------------------
// Our default help function.
//
// Syntax in the game: .<module> <arguments>
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandHelp(char** argv, int argc)
{
	// If command line is longer than 2, show usage for 'help'
	if (argc>2) return FALSE;

	char t[1024];
	sprintf(t, "Available commands for %s:", argv[0]);
	server->GamePrintInfo(t);

	// Loop through ModuleCommands[] and print their names
	for (int i=0; ModuleCommands[i].szName != NULL; i++)
	{
		sprintf(t, "c4.%s %s", argv[0], ModuleCommands[i].szName);
		server->GamePrintInfo(t);
	}

	sprintf(t, "For help on a specific command, type c4.%s <command> help", argv[0]);
	server->GamePrintInfo(t);
	return TRUE;
}
